# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2007 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

DOCUMENT_CLASSES = {
	"article" 	: _("Article"),
	"report" 	: _("Report"),
	"book" 		: _("Book"),
	"beamer" 	: _("Beamer slides"),
	"letter" 	: _("Letter"),
	"scrartcl" 	: _("Article (KOMA-Script)"),
	"scrreport" : _("Report (KOMA-Script)"),
	"scrbook" 	: _("Book (KOMA-Script)"),
	"scrlettr" 	: _("Letter (KOMA-Script)"),
	"scrlttr2" 	: _("Letter 2 (KOMA-Script)")
}

PAPER_SIZES = {
	"a4paper" 			: "A4",
	"a5paper" 			: "A5",
	"b5paper" 			: "B5",
	"executivepaper" 	: "US-Executive",
	"legalbl_paper" 	: "US-Legal",
	"letterpaper" 		: "US-Letter"
}

INPUT_ENCODINGS = {
	"utf8" : "UTF-8 (Unicode)",
	"ascii" : "US-ASCII",
	"next" : "ASCII (NeXT)",
	"ansinew" : "ASCII (Windows)",
	"applemac" : "ASCII (Apple)",
	"macce" : "MacCE (Apple Central European)",
	"latin1" : "Latin-1",
	"latin2" : "Latin-2",
	"latin3" : "Latin-3 (South European)",
	"latin4" : "Latin-4 (North European)",
	"latin5" : "Latin-5 (Turkish)",
	"latin6" : "Latin-6 (Nordic)",
	"latin7" : "Latin-7 (Baltic)",
	"latin8" : "Latin-8 (Celtic)",
	"latin9" : "Latin-9 (extended Latin-1)",
	"latin10" : "Latin-10 (South-Eastern European)",
	"cp1250" : "CP1250 (Windows Central European)",
	"cp1252" : "CP1252 (Windows Western European)",
	"cp1257" : "CP1257 (Windows Baltic)",
	"cp437" : "CP437 (DOS US)",
	"cp850" : "CP850 (DOS Latin-1)",
	"cp852" : "CP852 (DOS Central European)",
	"cp858" : "CP858 (DOS Western European)",
	"cp865" : "CP865 (DOS Nordic)"
}

BABEL_PACKAGES = {
	"afrikaans" : "Afrikaans",
	"american" : "American",
	"athnum" : "Athnum",
	"austrian" : "Austrian",
	"naustrian" : "Austrian (new spelling)",
	"bahasa" : "Bahasa",
	"basque" : "Basque",
	"breton" : "Breton",
	"british" : "British",
	"bulgarian" : "Bulgarian",
	"catalan" : "Catalan",
	"croatian" : "Croatian",
	"czech" : "Czech",
	"danish" : "Danish",
	"dutch" : "Dutch",
	"english" : "English",
	"UKenglish" : "English (UK)",
	"USenglish" : "English (US)",
	"esperanto" : "Esperanto",
	"estonian" : "Estonian",
	"finnish" : "Finnish",
	"francais" : "Francais",
	"galician" : "Galician",
	"german" : "German",
	"ngerman" : "German (new spelling)",
	"greek" : "Greek",
	"hebrew" : "Hebrew",
	"icelandic" : "Icelandic",
	"interlingua" : "Interlingua",
	"irish" : "Irish",
	"italian" : "Italian",
	"latin" : "Latin",
	"lsorbian" : "Lsorbian",
	"magyar" : "Magyar",
	"norsk" : "Norsk",
	"polish" : "Polish",
	"portuges" : "Portuges",
	"romanian" : "Romanian",
	"russianb" : "Russian",
	"samin" : "Samin",
	"scottish" : "Scottish",
	"serbian" : "Serbian",
	"slovak" : "Slovak",
	"slovene" : "Slovene",
	"spanish" : "Spanish",
	"swedish" : "Swedish",
	"turkish" : "Turkish",
	"ukraineb" : "Ukraine",
	"usorbian" : "Usorbian",
	"welsh" : "Welsh"
}

BEAMER_THEMES = ['default', 'Rochester', 'Szeged', 'Darmstadt', 'compatibility', 
	'shadow', 'Hannover', 'Luebeck', 'Madrid', 'sidebar', 'Montpellier', 'Warsaw', 
	'Ilmenau', 'Malmoe', 'Berlin', 'Berkeley', 'lined', 'bars', 'split', 'tree', 
	'Bergen', 'classic', 'PaloAlto', 'Pittsburgh', 'Antibes', 'Dresden', 'plain', 
	'Copenhagen', 'boxes', 'Frankfurt', 'Singapore', 'Goettingen', 'Marburg', 
	'Boadilla', 'JuanLesPins']

LOCALE_MAPPINGS = {
	"en_US" : "american",
	"en_AU" : "english",
	"fr" 	: "french",
	"it" 	: "italian",
	"ru" 	: "russian",
	"de_DE" : "ngerman",
	"de_AU" : "naustrian"
}

from gtk import Dialog, DIALOG_MODAL, DIALOG_DESTROY_WITH_PARENT, STOCK_OK, STOCK_CANCEL, RESPONSE_OK, \
	RESPONSE_CANCEL, Label, Entry, combo_box_new_text, Adjustment, SpinButton, HBox, Table, \
	CheckButton, VBox, Expander, RadioButton, ListStore, ComboBoxEntry
from locale import getdefaultlocale, nl_langinfo, D_FMT
from getpass import getuser
from pwd import getpwnam
from time import strftime
from os import popen, system
from os.path import splitext, basename
from string import Template

from Installation import Installation
from UnitComboBox import UnitComboBox, CENTIMETER


#class Environment(object):
#	def __init__(self):
#		self.__kpse_checked = False
#		self.__kpse_installed = None
#		
#		self.__document_classes = []
#		self.__babel_packages = []
#		self.__input_encodings = []
#		self.__username = []
#	
#	def __is_kpsewhich_installed(self):
#		"""
#		Check whether 'kpse' is installed. It is used for checking for available packages.
#		"""
#		if not self.__kpse_checked:
#			self.__kpse_installed = bool(system("kpsewhich --version $2>/dev/null") == 0)
#			self.__kpse_checked = True
#		
#		return self.__kpse_installed
#	
#	def get_document_classes(self):
#		pass
#	
#	def get_input_encodings(self):
#		pass
#	
#	def get_babel_packages(self):
#		pass
#	
#	def get_username(self):
#		pass
#	
#	def get_localized_date(self):
#		pass


#class _QuickstartDialog(Dialog):
#	def __init__(self, parent):
#		Dialog.__init__(self, "New LaTeX Document", parent, 
#					DIALOG_MODAL | DIALOG_DESTROY_WITH_PARENT, 
#					(STOCK_CANCEL, RESPONSE_CANCEL, STOCK_OK, RESPONSE_OK))
#		
#		store_classes = ListStore(str, str)
#		store_classes.append(["article", "Article"])
#		store_classes.append(["scrartcl", "Article (KOMA-Script)"])
#		
#		cmb_classes = ComboBoxEntry(store_classes, 0)
#		
#		self.vbox.add(cmb_classes)
#		self.vbox.show_all()
#	
#	def __get_cmb_classes(self):
##		self.cmb_delims = ComboBox(self.__store_delims)
##		cellPixbuf = CellRendererPixbuf()
##  		self.cmb_delims.pack_start(cellPixbuf, False)
##  		self.cmb_delims.add_attribute(cellPixbuf, 'pixbuf', 0)
##		cellText = CellRendererText()
##  		self.cmb_delims.pack_start(cellText, False)
##  		self.cmb_delims.add_attribute(cellText, 'text', 1)
##  		self.cmb_delims.set_active(0)
##  		self.cmb_delims.set_sensitive(False)
#		
#		pass
#	
#
#if __name__=="__main__":
#	dialog = _QuickstartDialog(None)
#	dialog.run()
#	import gtk
#	gtk.main()


class QuickstartDialog(Dialog):
	def __load_user_info(self):
		self.lang_code = getdefaultlocale()[0]		# store language code
		self.date_format = nl_langinfo(D_FMT)		# store localized date format for use in strftime()
		record = getpwnam(getuser())				# get pwd entry
		
		if len(record[4]):
			self.username = record[4].split(",")[0]
		else:
			self.username = record[0].title()
	
	def __kpsewhich_present(self):
		return bool(system("kpsewhich --version $2>/dev/null") == 0)
	
	def __scan_environment(self):
		"""
		Use kpsewhich to search for available document classes, input encodings,
		babel packages and beamer themes
		"""
		if self.__kpsewhich_present():
			self.document_classes = self.__getAvailable(DOCUMENT_CLASSES.keys(), "", ".cls")
			self.input_encodings = self.__getAvailable(INPUT_ENCODINGS.keys(), "", ".def")
			self.babel_packages = self.__getAvailable(BABEL_PACKAGES.keys(), "", ".sty")
			if "beamer" in self.document_classes:
				self.BEAMER_THEMES = self.__getAvailable(BEAMER_THEMES, "beamertheme", ".sty")
			else:
				self.BEAMER_THEMES = []
		else:
			self.document_classes = DOCUMENT_CLASSES.keys()
			self.input_encodings = INPUT_ENCODINGS.keys()
			self.babel_packages = BABEL_PACKAGES.keys()
			self.BEAMER_THEMES = BEAMER_THEMES
		self.document_classes.sort()
		self.input_encodings.sort()
		self.babel_packages.sort()
		
		self.paper_sizes = PAPER_SIZES.keys()
		self.paper_sizes.sort()
	
	def __init__(self, instance):
		self.instance = instance
		
		self.__load_user_info()
		self.__scan_environment()
		
		Dialog.__init__(self, _("New LaTeX Document"), instance.window, DIALOG_MODAL | DIALOG_DESTROY_WITH_PARENT, (STOCK_CANCEL, RESPONSE_CANCEL, STOCK_OK, RESPONSE_OK))
		
		self.set_border_width(5)
		self.set_has_separator(False)
		self.set_resizable(False)
		self.vbox.set_spacing(5)
		
		## General
		lbl_general = Label()
		lbl_general.set_markup("<b>General</b>")
		lbl_general.set_alignment(0, .5)
		
		self.entry_title = Entry()
		self.entry_title.set_activates_default(True)
		
		self.entry_author = Entry()
		self.entry_author.set_activates_default(True)
		
		self.entry_keywords = Entry()
		self.entry_keywords.set_activates_default(True)
		
		lbl_title = Label(_("_Title:"))
		lbl_title.set_mnemonic_widget(self.entry_title)
		
		lbl_author = Label(_("_Author:"))
		lbl_author.set_mnemonic_widget(self.entry_author)
		
		lbl_date = Label("Date:")
		#lbl_date.set_mnemonic_widget(self.entry_date)
		
		self.rad_today = RadioButton(label="Today")
		self.rad_cdate = RadioButton(self.rad_today, "Custom date:")
		
		self.entry_date = Entry()
		self.entry_date.set_activates_default(True)
		
		box_date = HBox(spacing=5)
		box_date.pack_start(self.rad_today)
		box_date.pack_start(self.rad_cdate)
		box_date.pack_start(self.entry_date)
		
		lbl_keywords = Label(_("_Keywords:"))
		lbl_keywords.set_mnemonic_widget(self.entry_keywords)
		
		for label in [lbl_title, lbl_author, lbl_date, lbl_keywords]:
			label.set_use_underline(True)
			label.set_alignment(0, .5)
		
		self.check_meta = CheckButton(_("Include PDF _metadata"))
		self.check_meta.set_use_underline(True)
		self.check_meta.set_active(True)
		
		## Style and Layout
		lbl_style = Label()
		lbl_style.set_markup("<b>Style and Layout</b>")
		lbl_style.set_alignment(0, .5)
		
		# document class
		self.cmb_class = combo_box_new_text()
		for name in self.document_classes:
			self.cmb_class.append_text(DOCUMENT_CLASSES[name])
		
		# beamer theme
		self.cmb_theme = combo_box_new_text()
		for name in self.BEAMER_THEMES:
			self.cmb_theme.append_text(name)
		
		# font size
		self.adj_default = Adjustment(10, 10, 12, 1)
		self.adj_beamer = Adjustment(11, 8, 20, 1)
		self.spin_fontsize = SpinButton()
		box_fontsize = HBox()
		box_fontsize.set_spacing(5)
		box_fontsize.pack_start(self.spin_fontsize, False)
		lbl_fontsize_unit = Label("pt")
		box_fontsize.pack_start(lbl_fontsize_unit, False)
		
		# paper size
		self.cmb_paper_size = combo_box_new_text()
		for name in self.paper_sizes:
			self.cmb_paper_size.append_text(PAPER_SIZES[name])
			
		# paper alignment
		self.cmb_paper_align = combo_box_new_text()
		for item in [_("Portrait"), _("Landscape")]:
			self.cmb_paper_align.append_text(item)
		
		lbl_class = Label(_("Document _Class:"))
		lbl_class.set_mnemonic_widget(self.cmb_class)
		self.lbl_theme = Label(_("Th_eme:"))
		self.lbl_theme.set_mnemonic_widget(self.cmb_theme)
		lbl_font_size = Label(_("Font _Size:"))
		lbl_font_size.set_mnemonic_widget(self.spin_fontsize)
		lbl_paper = Label(_("_Paper:"))
		lbl_paper.set_mnemonic_widget(self.cmb_paper_size)
		
		for label in [lbl_class, self.lbl_theme, lbl_font_size, lbl_fontsize_unit, lbl_paper]:
			label.set_use_underline(True)
			label.set_alignment(0, .5)
		
		## Language Customization
		lbl_language = Label()
		lbl_language.set_markup("<b>Language Customization</b>")
		lbl_language.set_alignment(0, .5)
		
		# input encoding
		self.cmb_encoding = combo_box_new_text()
		for name in self.input_encodings:
			self.cmb_encoding.append_text(INPUT_ENCODINGS[name])
		
		lbl_encoding = Label(_("_Input encoding:"))
		lbl_encoding.set_use_underline(True)
		lbl_encoding.set_mnemonic_widget(self.cmb_encoding)
		lbl_encoding.set_alignment(0, .5)
		
		# babel package
		self.cmb_babel = combo_box_new_text()
		for name in self.babel_packages:
			self.cmb_babel.append_text(BABEL_PACKAGES[name])
		
		lbl_babel = Label(_("_Babel package:"))
		lbl_babel.set_use_underline(True)
		lbl_babel.set_mnemonic_widget(self.cmb_babel)
		lbl_babel.set_alignment(0, .5)
		
		tbl_language = Table()
		tbl_language.set_row_spacings(5)
		tbl_language.set_col_spacings(5)
		tbl_language.attach(lbl_encoding, 1, 2, 0, 1)
		tbl_language.attach(self.cmb_encoding, 2, 3, 0, 1)
		tbl_language.attach(lbl_babel, 1, 2, 1, 2)
		tbl_language.attach(self.cmb_babel, 2, 3, 1, 2)
		tbl_language.set_col_spacing(0, 15)
		
		exp_language = Expander(_("<b>_Language Customization</b>"))
		exp_language.set_use_markup(True)
		exp_language.set_use_underline(True)
		exp_language.add(tbl_language)
		
		## Margins
		self.lbl_m_top = Label(_("Top:"))
		self.lbl_m_right = Label(_("Right:"))
		self.lbl_m_bottom = Label(_("Bottom:"))
		self.lbl_m_left = Label(_("Left:"))
		
		self.spin_m_top = SpinButton(Adjustment(2.5, 0, 1000, 1))
		self.spin_m_right = SpinButton(Adjustment(2.5, 0, 1000, 1))
		self.spin_m_bottom = SpinButton(Adjustment(2.5, 0, 1000, 1))
		self.spin_m_left = SpinButton(Adjustment(2.5, 0, 1000, 1))
		
		self.spin_m_top.set_digits(2)
		self.spin_m_right.set_digits(2)
		self.spin_m_bottom.set_digits(2)
		self.spin_m_left.set_digits(2)
		
		self.check_margins = CheckButton(_("C_ustomize Margins:"))
		self.check_margins.set_use_underline(True)
		
		self.cmb_margin_unit = UnitComboBox(CENTIMETER)
		for spinbutton in [self.spin_m_top, self.spin_m_right, self.spin_m_bottom, self.spin_m_left]:
			self.cmb_margin_unit.attach_spinbutton(spinbutton)
		
		## Paragraph Indent
		self.spin_indent = SpinButton(Adjustment(1.0, 0, 1000, 1))
		self.spin_indent.set_digits(2)
		
		self.cmb_indent_unit = UnitComboBox(CENTIMETER)
		self.cmb_indent_unit.attach_spinbutton(self.spin_indent)
		
		self.check_indent = CheckButton(_("Set Pa_ragraph Indentation:"))
		self.check_indent.set_use_underline(True)
		
		box_indent = HBox()
		box_indent.set_spacing(5)
		box_indent.pack_start(self.spin_indent, False)
		box_indent.pack_start(self.cmb_indent_unit, False)
		
		tbl_margins = Table()
		tbl_margins.set_row_spacings(5)
		tbl_margins.set_col_spacings(5)
		tbl_margins.attach(self.check_margins, 1, 11, 0, 1)
		tbl_margins.attach(self.lbl_m_top, 2, 3, 1, 2)
		tbl_margins.attach(self.spin_m_top, 3, 4, 1, 2)
		tbl_margins.attach(self.lbl_m_right, 4, 5, 1, 2)
		tbl_margins.attach(self.spin_m_right, 5, 6, 1, 2)
		tbl_margins.attach(self.lbl_m_bottom, 6, 7, 1, 2)
		tbl_margins.attach(self.spin_m_bottom, 7, 8, 1, 2)
		tbl_margins.attach(self.lbl_m_left, 8, 9, 1, 2)
		tbl_margins.attach(self.spin_m_left, 9, 10, 1, 2)
		tbl_margins.attach(self.cmb_margin_unit, 10, 11, 1, 2)
		tbl_margins.attach(self.check_indent, 1, 11, 2, 3)
		tbl_margins.attach(box_indent, 2, 11, 3, 4)
		tbl_margins.set_col_spacing(0, 15)
		tbl_margins.set_col_spacing(1, 15)
		
		exp_margins = Expander(_("<b>_Margins</b>"))
		exp_margins.set_use_markup(True)
		exp_margins.set_use_underline(True)
		exp_margins.add(tbl_margins)
		
		## Pack
		table = Table()
		
		table.set_border_width(5)
		table.set_row_spacings(5)
		table.set_col_spacings(5)
		
		table.attach(lbl_general, 0, 4, 0, 1)
		table.attach(lbl_title, 1, 2, 1, 2)
		table.attach(self.entry_title, 2, 4, 1, 2)
		table.attach(lbl_author, 1, 2, 2, 3)
		table.attach(self.entry_author, 2, 4, 2, 3)
		table.attach(lbl_date, 1, 2, 3, 4)
		table.attach(box_date, 2, 4, 3, 4)
		table.attach(lbl_keywords, 1, 2, 4, 5)
		table.attach(self.entry_keywords, 2, 4, 4, 5)
		table.attach(self.check_meta, 1, 4, 5, 6)
		
		table.attach(lbl_style, 0, 4, 6, 7)
		table.attach(lbl_class, 1, 2, 7, 8)
		table.attach(self.cmb_class, 2, 4, 7, 8)
		table.attach(self.lbl_theme, 1, 2, 8, 9)
		table.attach(self.cmb_theme, 2, 4, 8, 9)
		table.attach(lbl_font_size, 1, 2, 9, 10)
		table.attach(box_fontsize, 2, 4, 9, 10)
		table.attach(lbl_paper, 1, 2, 10, 11)
		table.attach(self.cmb_paper_size, 2, 3, 10, 11)
		table.attach(self.cmb_paper_align, 3, 4, 10, 11)
		
		table.attach(exp_language, 0, 4, 11, 12)
		table.attach(exp_margins, 0, 4, 12, 13)
		
		table.set_col_spacing(0, 15)
		table.set_row_spacing(5, 15)
		table.set_row_spacing(10, 15)
		table.set_row_spacing(11, 15)
		
		self.vbox.add(table)
		
		self.connect("response", self.__on_response)
		self.cmb_class.connect("changed", self.__on_class_changed)
		self.check_margins.connect("toggled", self.__on_margins_toggled)
		self.check_indent.connect("toggled", self.__on_indent_toggled)
		
		self.set_default_response(RESPONSE_OK)
		
		self.check_margins.set_active(True)
		self.check_indent.set_active(True)
		self.check_margins.set_active(False)
		self.check_indent.set_active(False)
	
	def __on_margins_toggled(self, check_button):
		x = check_button.get_active()
		for widget in [self.lbl_m_top, self.lbl_m_right, self.lbl_m_bottom, self.lbl_m_left, \
				self.spin_m_top, self.spin_m_right, self.spin_m_bottom, self.spin_m_left, self.cmb_margin_unit]:
			widget.set_sensitive(x)
	
	def __on_indent_toggled(self, check_button):
		x = check_button.get_active()
		self.spin_indent.set_sensitive(x)
		self.cmb_indent_unit.set_sensitive(x)
	
	def run(self):
		self.__load_settings()
		self.show_all()
		self.set_default_response(RESPONSE_OK)
		
		return Dialog.run(self)
		
	def __load_settings(self):
		""" 
		Restore settings from configuration
		"""
		self.entry_author.set_text(self.username)
		self.entry_date.set_text(strftime(self.date_format))
		
		## document class
		lastClass = self.instance.settings.setdefault("last-document-class", self.document_classes[0])
		try:
			index = self.document_classes.index(lastClass)
		except ValueError:
			index = 0
		self.cmb_class.set_active(index)
		
		## beamer theme
		try:
			lastTheme = self.instance.settings.setdefault("last-beamer-theme", self.BEAMER_THEMES[0])
		except IndexError:
			lastTheme = ""
			
		try:
			index = self.BEAMER_THEMES.index(lastTheme)
		except ValueError:
			index = 0
		self.cmb_theme.set_active(index)
		
		## paper size
		last_size = self.instance.settings.setdefault("last-paper-size", self.paper_sizes[0])
		try:
			index = self.paper_sizes.index(last_size)
		except ValueError:
			index = 0
		self.cmb_paper_size.set_active(index)
		
		## paper alignment
		self.cmb_paper_align.set_active(int(self.instance.settings.setdefault("last-paper-alignment", 0)))
		
		## input encoding
		try:
			last_encoding = self.instance.settings["last-input-encoding"]
			# restore the last selected input encoding
			try:
				index = self.input_encodings.index(last_encoding)
			except ValueError:
				index = 0
			self.cmb_encoding.set_active(index)
		except KeyError:
			# use Unicode
			try:
				index = self.input_encodings.index("utf8")
			except (IndexError, ValueError):
				index = 0
			self.cmb_encoding.set_active(index)
		
		# babel package
		try:
			last_babel = self.instance.settings["last-babel-package"]
			# restore the last selected babel package
			try:
				index = self.babel_packages.index(last_babel)
			except ValueError:
				index = 0
			self.cmb_babel.set_active(index)
				
		except KeyError:
			# guess a babel package from the user's locale settings
			try:
				babel = LOCALE_MAPPINGS[self.lang_code]
				index = self.babel_packages.index(babel)
			except (KeyError, IndexError, ValueError):
				print "Warning: could not map your locale '%s' to a babel package." % self.lang_code
				index = 0
			self.cmb_babel.set_active(index)
		
	def __on_response(self, dialog, response):
		if response == RESPONSE_OK:
			# store settings
			if len(self.BEAMER_THEMES):
				self.instance.settings["last-beamer-theme"] = self.BEAMER_THEMES[self.cmb_theme.get_active()]
				
			self.instance.settings["last-document-class"] = self.document_classes[self.cmb_class.get_active()]
			self.instance.settings["last-paper-size"] = self.paper_sizes[self.cmb_paper_size.get_active()]
			self.instance.settings["last-paper-alignment"] = int(self.cmb_paper_align.get_active())
			self.instance.settings["last-input-encoding"] = self.input_encodings[self.cmb_encoding.get_active()]
			self.instance.settings["last-babel-package"] = self.babel_packages[self.cmb_babel.get_active()]
	
	def __on_class_changed(self, combo):
		documentClass = self.document_classes[self.cmb_class.get_active()]
		setting = (documentClass == "beamer")
		# theme
		self.lbl_theme.set_sensitive(setting)
		self.cmb_theme.set_sensitive(setting)
		# font size
		if documentClass == "beamer":
			self.spin_fontsize.set_adjustment(self.adj_beamer)
		else:
			self.spin_fontsize.set_adjustment(self.adj_default)
			if documentClass in ["article", "report", "book", "letter"]:
				self.spin_fontsize.set_value(10)
			else:
				self.spin_fontsize.set_value(11)
	
	@property
	def source(self):
		""" 
		Returns the source corresponding to the settings made 
		"""
		document_class = self.document_classes[self.cmb_class.get_active()]
		
		## escape strings
		title = self.entry_title.get_text().replace("&", r"\&")
		author = self.entry_author.get_text().replace("&", r"\&")
		
		if self.rad_today.get_active():
			date = "\\today"
		else:
			date = self.entry_date.get_text().replace("&", r"\&")
		
		d = {
			"class" 	: document_class,
			"options" 	: "%spt,%s" % (self.spin_fontsize.get_value_as_int(), self.paper_sizes[self.cmb_paper_size.get_active()]),
			"inputenc" 	: "\\usepackage[%s]{inputenc}\n" % self.input_encodings[self.cmb_encoding.get_active()],
			"babel" 	: "\\usepackage[%s]{babel}\n" % self.babel_packages[self.cmb_babel.get_active()],
			"title" 	: title,
			"author" 	: author,
			"date" 		: date
		}
		
		## PDF metadata
		if self.check_meta.get_active():
			keywords = self.entry_keywords.get_text().replace("&", r"\&")
			d["ifpdf"] = "\\usepackage{ifpdf}\n"
			d["pdfmeta"] = "\\ifpdf\n\\pdfinfo {\n\t/Author (%s)\n\t/Title (%s)\n\t/Subject (%s)\n\t/Keywords (%s)\n\t/CreationDate (D:%s)\n}\n\\fi\n" % \
				(author, title, title, keywords, strftime('%Y%m%d%H%M%S'))
		else:
			d["ifpdf"] = ""
			d["pdfmeta"] = ""
		
		## beamer theme
		if len(self.BEAMER_THEMES):
			d["theme"] = self.BEAMER_THEMES[self.cmb_theme.get_active()]
		
		## paper align
		if self.cmb_paper_align.get_active():
			d["options"] += ",landscape"
		
		## margins
		if self.check_margins.get_active():
			u = self.cmb_margin_unit.get_unit_string()
			d["geometry"] = "\\usepackage[top=%s%s,right=%s%s,bottom=%s%s,left=%s%s]{geometry}\n" % (
					round(self.spin_m_top.get_value(), 2), u, 
					round(self.spin_m_right.get_value(), 2), u, 
					round(self.spin_m_bottom.get_value(), 2), u, 
					round(self.spin_m_left.get_value(), 2), u)
		else:
			d["geometry"] = ""
		
		## paragraph indentation
		if self.check_indent.get_active():
			d["parindent"] = "\\parindent %s%s\n" % (self.spin_indent.get_value(), self.cmb_indent_unit.get_unit_string())
		else:
			d["parindent"] = ""
		
		try:
			template = open("%s/templates/%s" % (Installation().path, document_class)).read()
		except IOError:
			template = open(Installation().path + "/templates/default").read()
			
		source = Template(template).safe_substitute(d)
		return tuple(source.split("$cursor"))
	
	def __getAvailable(self, names, prefix="", suffix=""):
		""" 
		Builds filenames from names, prefix and suffix and returns only the
		available of them 
		"""
		# TODO: use generators
		filenames = ["%s%s%s" % (prefix, name, suffix) for name in names]
		foundFilenames = popen("kpsewhich " + " ".join(filenames)).read().splitlines()
		foundNames = [splitext(basename(filename))[0][len(prefix):] for filename in foundFilenames]
		
		return foundNames
		
