# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2007 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

from gtk import Window, TEXT_WINDOW_WIDGET, WINDOW_POPUP, Label, \
		CellRendererText, ScrolledWindow, POLICY_AUTOMATIC, Frame, SHADOW_OUT
from gtk.gdk import screen_width, screen_height

class HintPopup(Window):
	"""
	A popup showing a hint to the current argument
	"""
	
	def __init__(self, window):
		Window.__init__(self, WINDOW_POPUP)
		
		self.__window = window
		
		self.__label = Label()
		self.__label.set_use_markup(True)

		self.__frame = Frame()
		self.__frame.set_shadow_type(SHADOW_OUT)
		self.__frame.add(self.__label)
		
		self.add(self.__frame)
		
		self.__active = False
	
	def set_hint(self, hint):
		self.__label.set_markup(hint)
		
		# FIXME: this has no effect
		self.__label.queue_resize()
		self.__frame.queue_resize()
		self.queue_resize()
	
	def set_active(self, active):
		"""
		Shows or hides the popup
		"""
		if active and not self.__active:
			self.show_all()
		elif not active and self.__active:
			self.hide()
		
		self.__active = active
	
	def get_active(self):
		return self.__active

	def __get_cursor_pos(self, text_view):
		text_buffer = text_view.get_buffer()
		insert_mark = text_buffer.get_insert()
		start = text_buffer.get_iter_at_mark(insert_mark)
		location = text_view.get_iter_location(start)
		win_x, win_y = text_view.buffer_to_window_coords(TEXT_WINDOW_WIDGET, location.x, location.y)
		
		win = text_view.get_window(TEXT_WINDOW_WIDGET)
		xx, yy = win.get_origin()
		
		x = win_x + xx
		#y = win_y + yy + location.height
		y = win_y + yy
		
		return (x, y)
	
	def move_to_cursor(self, text_view):
		
		x = 0
		y = 0
	
		sw = screen_width()
		sh = screen_height()
		
		x, y = self.__get_cursor_pos(text_view)
		
		w, h = self.get_size()
		
		if x + w > sw:
			x = sw - w - 4
		
#		if y + h > sh:
#			layout = text_view.create_pango_layout()
#			xtext, ytext = layout.get_pixel_size()
#			y = y - ytext - h
		
		# TODO: really get the height of the popup
		self.move(x, y - 24)
		