# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2007 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

from re import compile

from Token import Token

class PrefixScanner(object):
	"""
	A very simple LaTeX scanner
	"""
	def __init__(self):
		self.__pattern = compile(r"(?P<backslash>\\)|(?P<curlyopen>\{)|(?P<curlyclose>\})|(?P<squareopen>\[)|(?P<squareclose>\])|(?P<roundopen>\()|(?P<roundclose>\))|(?P<comma>\,)|(?P<name>[a-zA-Z]+[\*]?)|(?P<other>[^a-zA-Z\\\{\}\[\]\(\)\,])")
		self.__match_iter = None
	
	def set_prefix(self, prefix):
		self.__match_iter = self.__pattern.finditer(prefix)
		self.__end = False
	
	def __iter__(self):
		return self
	
	def next(self):
		if self.__end:
			raise StopIteration
		
		try:
			match = self.__match_iter.next()
		except StopIteration:
			self.__end = True
			return Token(Token.T_END)
		
		if match.group("backslash"):
			return Token(Token.T_BACKSLASH)
		elif match.group("curlyopen"):
			return Token(Token.T_CURLY_OPEN)
		elif match.group("curlyclose"):
			return Token(Token.T_CURLY_CLOSE)
		elif match.group("squareopen"):
			return Token(Token.T_SQUARE_OPEN)
		elif match.group("squareclose"):
			return Token(Token.T_SQUARE_CLOSE)
		elif match.group("roundopen"):
			return Token(Token.T_ROUND_OPEN)
		elif match.group("roundclose"):
			return Token(Token.T_ROUND_CLOSE)
		elif match.group("comma"):
			return Token(Token.T_COMMA)
		elif match.group("name"):
			return Token(Token.T_NAME, match.group("name"))
		elif match.group("other"):
			return Token(Token.T_OTHER, match.group("other"))
		