# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2007 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

from xml.sax.saxutils import escape

from BibTeXNode import BibTeXNode
from BibTeXMultipleValues import BibTeXMultipleValues

class BibTeXEntry(BibTeXNode):
	def __init__(self, line, column, type, key):
		BibTeXNode.__init__(self, line, column)
		
		self.line = line
		self.column = column
		
		self.type = type
		self.key = key
		
		self.__fields = {}
		
	def get_field_value(self, name):
		return self.__fields[name]
	
	def add_field_value(self, field_name, value):
		assert field_name != None
		assert value != None
		assert not type(value) is BibTeXMultipleValues
		
		try:
			current_value = self.get_field_value(field_name)
			
			if type(current_value) is BibTeXMultipleValues:
				current_value.add_value(value)
			else:
				multiple_values = self.get_owner_file().make_multiple_values()
				multiple_values.add_value(current_value)
				multiple_values.add_value(value)
				self.set_field(field_name, multiple_values)
		except KeyError:
			self.set_field_value(field_name, value)
	
	def set_field(self, field_name, field_value):
		assert field_name != None
		assert field_value != None
		
		self.__fields[field_name.lower()] = field_value
	
	@property
	def fields(self):
		return self.__fields

	@property
	def markup(self):
		"""
		Called by the code completion
		"""
		s = []
		for name, value in self.fields.iteritems():
			s.append("<small><b>%s</b></small> %s\n" % (escape(name), escape(str(value))))
		return "".join(s)
		