# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2007 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

from os.path import dirname, basename, exists
from xml.dom.minidom import parse, getDOMImplementation
from xml.parsers.expat import ExpatError

class MetaFile(dict):
	"""
	This represents a hidden file containing metadata for a LaTeX file
	"""
	def __init__(self, source_filename):
		"""
		Construct a MetaFile for a LaTeX source file
		"""
		dict.__init__(self)
		
		self.__source_filename = source_filename
		self.__filename = "%s/.%s.meta.xml" % (dirname(source_filename), basename(source_filename))
		
		try:
			self.__dom = parse(self.__filename)
			
			for property_node in self.__dom.getElementsByTagName("property"):
				k = property_node.getAttribute("key")
				v = property_node.getAttribute("value")
				self.__setitem__(k, v)
				
		except IOError:
			print "File %s not found, creating empty one..." % self.__filename
			
			self.__dom = getDOMImplementation().createDocument(None, "properties", None)
		
		except ExpatError, e:
			print "Error parsing %s: %s" % (self.__filename, e)
		
	
	def __get_node(self, k):
		for node in self.__dom.getElementsByTagName("property"):
			if node.getAttribute("key") == str(k):
				return node
		raise KeyError
	
	def __getitem__(self, k):
		node = self.__get_node(k)
		return node.getAttribute("value")
	
	def __setitem__(self, k, v):
		try:
			self.__get_node(k).setAttribute("value", str(v))
		except KeyError:
			node = self.__dom.createElement("property")
			node.setAttribute("key", str(k))
			node.setAttribute("value", str(v))
			self.__dom.documentElement.appendChild(node)
	
	def save(self):
		if exists(self.__filename):
			mode = "w"
		else:
			mode = "a"
		
		try:
			f = open(self.__filename, mode)
			f.write(self.__dom.toxml())
			f.close()
		except IOError, e:
			print "Error saving %s: %s" % (self.__filename, e)



if __name__ == "__main__":
	m = MetaFile("/home/michael/LaTeXProjekt/slave.tex")
	#m["master"] = "/home/michael/LaTeXProjekt/master.tex"
	print m["LatexMasterFilename"]
	m.save()
	
	
	
	